<?php

namespace Drupal\emoji_picker\Hook;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Template\Attribute;

/**
 * Alter the widget based on the third party settings.
 */
#[Hook('field_widget_complete_form_alter')]
class WidgetAlter {

  use StringTranslationTrait;

  /**
   * Inject services.
   *
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   *   Language Manager.
   */
  public function __construct(protected LanguageManagerInterface $languageManager) {
  }

  /**
   * Implements hook_field_widget_complete_form_alter().
   *
   * @param array $field_widget_complete_form
   *   The field widget form element.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param array $context
   *   An associative array containing the following key-value pairs:
   *     form: The form structure to which widgets are being attached.
   *     widget: The widget plugin instance.
   *     items: The field values, a FieldItemListInterface object.
   *     delta: The order of this item in the array of subelements.
   *     default: boolean. whether the form is to set default values.
   */
  public function __invoke(&$field_widget_complete_form, FormStateInterface $form_state, $context): void {
    $enable = $context['widget']->getThirdPartySetting('emoji_picker', 'enable');
    if (!$enable) {
      return;
    }
    $field_widget_complete_form['#attached']['library'][] = 'emoji_picker/emoji_picker';

    $attributes = new Attribute();

    if ($context['widget']->getThirdPartySetting('emoji_picker', 'restrict_input')) {
      $attributes->setAttribute('data-drupal-restrict-input', 'true');
      foreach (Element::children($field_widget_complete_form['widget']) as $delta) {
        if (
          !isset($field_widget_complete_form['widget'][$delta]['value']) ||
          $field_widget_complete_form['widget'][$delta]['value']['#type'] !== 'textfield'
        ) {
          continue;
        }
        $field_widget_complete_form['widget'][$delta]['value']['#element_validate'][] = [
          $this, 'emojiPickerFieldValidate',
        ];
      }
    }

    if (!$context['widget']->getThirdPartySetting('emoji_picker', 'display_picker')) {
      $attributes->addClass('visually-hidden');
    }

    $currentLanguage = $this->languageManager->getCurrentLanguage()->getId();
    if ($currentLanguage != 'en') {
      $attributes->setAttribute('locale', $currentLanguage);
    }

    $field_widget_complete_form['widget']['emoji_picker'] = [
      '#theme' => 'emoji_picker',
      '#attributes' => $attributes,
      '#display_picker' => $context['widget']->getThirdPartySetting('emoji_picker', 'display_picker'),
    ];
  }

  /**
   * Validate callback.
   */
  public function emojiPickerFieldValidate($element, FormStateInterface $form_state): void {
    $value = $form_state->getValue($element['#parents']);
    if (!empty(\Emoji\remove_emoji($value))) {
      $form_state->setError($element, $this->t('There is invalid emojis'));
    }
  }

}
